@@ -129,8 +129,8 @@ int proc_cmdline(char **ret) {
 
         if (detect_container() > 0)
                 return get_process_cmdline(1, SIZE_MAX, 0, ret);
-        else
-                return read_full_file("/proc/cmdline", ret, NULL);
+
+        return read_virtual_file("/proc/cmdline", SIZE_MAX, ret, NULL);
 }
 
 static int proc_cmdline_strv_internal(char ***ret, bool filter_pid1_args) {
