@@ -12,7 +12,7 @@ typedef enum MountExecCommand {
         MOUNT_EXEC_UNMOUNT,
         MOUNT_EXEC_REMOUNT,
         _MOUNT_EXEC_COMMAND_MAX,
-        _MOUNT_EXEC_COMMAND_INVALID = -1
+        _MOUNT_EXEC_COMMAND_INVALID = -EINVAL,
 } MountExecCommand;
 
 typedef enum MountResult {
@@ -25,7 +25,7 @@ typedef enum MountResult {
         MOUNT_FAILURE_START_LIMIT_HIT,
         MOUNT_FAILURE_PROTOCOL,
         _MOUNT_RESULT_MAX,
-        _MOUNT_RESULT_INVALID = -1
+        _MOUNT_RESULT_INVALID = -EINVAL,
 } MountResult;
 
 typedef struct MountParameters {
@@ -76,7 +76,6 @@ struct Mount {
         CGroupContext cgroup_context;
 
         ExecRuntime *exec_runtime;
-        DynamicCreds dynamic_creds;
 
         MountState state, deserialized_state;
 
