@@ -27,7 +27,7 @@ void pull_job_close_disk_fd(PullJob *j) {
         if (j->close_disk_fd)
                 safe_close(j->disk_fd);
 
-        j->disk_fd = -1;
+        j->disk_fd = -EBADF;
 }
 
 PullJob* pull_job_unref(PullJob *j) {
@@ -250,8 +250,8 @@ void pull_job_curl_on_finished(CurlGlue *g, CURL *curl, CURLcode result) {
                         if (j->offset == UINT64_MAX) {
 
                                 if (j->written_compressed > 0) {
-                                        /* Make sure the file size is right, in case the file was sparse and we just seeked
-                                         * for the last part */
+                                        /* Make sure the file size is right, in case the file was sparse and
+                                         * we just moved to the last part. */
                                         if (ftruncate(j->disk_fd, j->written_uncompressed) < 0) {
                                                 r = log_error_errno(errno, "Failed to truncate file: %m");
                                                 goto finish;
@@ -692,7 +692,7 @@ int pull_job_new(
 
         *j = (PullJob) {
                 .state = PULL_JOB_INIT,
-                .disk_fd = -1,
+                .disk_fd = -EBADF,
                 .close_disk_fd = true,
                 .userdata = userdata,
                 .glue = glue,
