@@ -4,6 +4,7 @@
 #include <linux/audit.h>
 
 #include "audit-type.h"
+#include "tests.h"
 
 static void print_audit_label(int i) {
         const char *name;
@@ -13,14 +14,11 @@ static void print_audit_label(int i) {
         printf("%i → %s → %s\n", i, audit_type_to_string(i), name);
 }
 
-static void test_audit_type(void) {
+TEST(audit_type) {
         int i;
 
         for (i = 0; i <= AUDIT_KERNEL; i++)
                 print_audit_label(i);
 }
 
-int main(int argc, char **argv) {
-        test_audit_type();
-        return 0;
-}
+DEFINE_TEST_MAIN(LOG_INFO);
