@@ -23,8 +23,8 @@
 #include "strv.h"
 #include "strxcpyx.h"
 
-#define ADDRESSES_PER_LINK_MAX 2048U
-#define STATIC_ADDRESSES_PER_NETWORK_MAX 1024U
+#define ADDRESSES_PER_LINK_MAX 16384U
+#define STATIC_ADDRESSES_PER_NETWORK_MAX 8192U
 
 int address_flags_to_string_alloc(uint32_t flags, int family, char **ret) {
         _cleanup_free_ char *str = NULL;
@@ -143,6 +143,14 @@ Address *address_free(Address *address) {
         return mfree(address);
 }
 
+static bool address_lifetime_is_valid(const Address *a) {
+        assert(a);
+
+        return
+                a->lifetime_valid_usec == USEC_INFINITY ||
+                a->lifetime_valid_usec > now(CLOCK_BOOTTIME);
+}
+
 bool address_is_ready(const Address *a) {
         assert(a);
 
@@ -158,7 +166,30 @@ bool address_is_ready(const Address *a) {
         if (!FLAGS_SET(a->state, NETWORK_CONFIG_STATE_CONFIGURED))
                 return false;
 
-        return true;
+        return address_lifetime_is_valid(a);
+}
+
+bool link_check_addresses_ready(Link *link, NetworkConfigSource source) {
+        Address *a;
+        bool has = false;
+
+        assert(link);
+
+        /* Check if all addresses on the interface are ready. If there is no address, this will return false. */
+
+        SET_FOREACH(a, link->addresses) {
+                if (source >= 0 && a->source != source)
+                        continue;
+                if (address_is_marked(a))
+                        continue;
+                if (!address_exists(a))
+                        continue;
+                if (!address_is_ready(a))
+                        return false;
+                has = true;
+        }
+
+        return has;
 }
 
 void link_mark_addresses(Link *link, NetworkConfigSource source) {
@@ -239,6 +270,19 @@ static void address_set_lifetime(Manager *m, Address *a, const struct ifa_cachei
         a->lifetime_preferred_usec = sec_to_usec(cinfo->ifa_prefered, now_usec);
 }
 
+static bool address_is_static_null(const Address *address) {
+        assert(address);
+
+        if (!address->network)
+                return false;
+
+        if (!address->requested_as_null)
+                return false;
+
+        assert(!in_addr_is_set(address->family, &address->in_addr));
+        return true;
+}
+
 static uint32_t address_prefix(const Address *a) {
         assert(a);
 
@@ -316,30 +360,6 @@ DEFINE_PRIVATE_HASH_OPS_WITH_KEY_DESTRUCTOR(
         address_kernel_compare_func,
         address_free);
 
-/* The functions below are mainly used by managing Request. */
-static void address_hash_func(const Address *a, struct siphash *state) {
-        assert(a);
-
-        siphash24_compress(&a->family, sizeof(a->family), state);
-
-        /* treat any other address family as AF_UNSPEC */
-        if (!IN_SET(a->family, AF_INET, AF_INET6))
-                return;
-
-        siphash24_compress(&a->prefixlen, sizeof(a->prefixlen), state);
-        siphash24_compress(&a->in_addr, FAMILY_ADDRESS_SIZE(a->family), state);
-        siphash24_compress(&a->in_addr_peer, FAMILY_ADDRESS_SIZE(a->family), state);
-
-        if (a->family == AF_INET) {
-                /* On update, the kernel ignores the address label and broadcast address, hence we need
-                 * to distinguish addresses with different labels or broadcast addresses. Otherwise,
-                 * the label or broadcast address change will not be applied when we reconfigure the
-                 * interface. */
-                siphash24_compress_string(a->label, state);
-                siphash24_compress(&a->broadcast, sizeof(a->broadcast), state);
-        }
-}
-
 int address_compare_func(const Address *a1, const Address *a2) {
         int r;
 
@@ -363,6 +383,10 @@ int address_compare_func(const Address *a1, const Address *a2) {
                 return r;
 
         if (a1->family == AF_INET) {
+                /* On update, the kernel ignores the address label and broadcast address, hence we need
+                 * to distinguish addresses with different labels or broadcast addresses. Otherwise,
+                 * the label or broadcast address change will not be applied when we reconfigure the
+                 * interface. */
                 r = strcmp_ptr(a1->label, a2->label);
                 if (r != 0)
                         return r;
@@ -386,16 +410,31 @@ int address_equal(const Address *a1, const Address *a2) {
 }
 
 static int address_equalify(Address *address, const Address *src) {
+        bool src_is_null;
         int r;
 
         assert(address);
         assert(src);
 
-        if (address_kernel_compare_func(address, src) != 0)
+        src_is_null = address_is_static_null(src);
+
+        if (src_is_null) {
+                /* When the source address has an null address, then we cannot compare the two objects with
+                 * address_kernel_compare_func(). Let's do minimal check here. */
+                if (address->family != src->family)
+                        return -EINVAL;
+                if (address->prefixlen != src->prefixlen)
+                        return -EINVAL;
+
+        } else if (address_kernel_compare_func(address, src) != 0)
                 return -EINVAL;
 
         if (address->family == AF_INET) {
-                address->broadcast = src->broadcast;
+                /* When the source is null, then the broadcast address will be determined based on the
+                 * acquired address. Hence, do not copy it here. */
+                if (!src_is_null)
+                        address->broadcast = src->broadcast;
+
                 r = free_and_strdup(&address->label, src->label);
                 if (r < 0)
                         return r;
@@ -549,18 +588,40 @@ static int address_drop(Address *address) {
 }
 
 int address_get(Link *link, const Address *in, Address **ret) {
-        Address *existing;
+        Address *a;
 
         assert(link);
         assert(in);
 
-        existing = set_get(link->addresses, in);
-        if (!existing)
-                return -ENOENT;
+        a = set_get(link->addresses, in);
+        if (a) {
+                if (ret)
+                        *ret = a;
+                return 0;
+        }
 
-        if (ret)
-                *ret = existing;
-        return 0;
+        /* Find matching address that originally requested as null address. */
+        if (address_is_static_null(in))
+                SET_FOREACH(a, link->addresses) {
+                        if (!a->requested_as_null)
+                                continue;
+
+                        /* Currently, null address is supported only by static addresses. Note that static
+                         * address may be set as foreign during reconfiguring the interface. */
+                        if (!IN_SET(a->source, NETWORK_CONFIG_SOURCE_FOREIGN, NETWORK_CONFIG_SOURCE_STATIC))
+                                continue;
+
+                        if (a->family != in->family)
+                                continue;
+                        if (a->prefixlen != in->prefixlen)
+                                continue;
+
+                        if (ret)
+                                *ret = a;
+                        return 0;
+                }
+
+        return -ENOENT;
 }
 
 int link_get_address(Link *link, int family, const union in_addr_union *address, unsigned char prefixlen, Address **ret) {
@@ -654,7 +715,7 @@ bool manager_has_address(Manager *manager, int family, const union in_addr_union
         if (manager_get_address(manager, family, address, 0, &a) < 0)
                 return false;
 
-        return check_ready ? address_is_ready(a) : address_exists(a);
+        return check_ready ? address_is_ready(a) : (address_exists(a) && address_lifetime_is_valid(a));
 }
 
 const char* format_lifetime(char *buf, size_t l, usec_t lifetime_usec) {
@@ -685,16 +746,22 @@ static void log_address_debug(const Address *address, const char *str, const Lin
         const char *peer = in_addr_is_set(address->family, &address->in_addr_peer) ?
                 IN_ADDR_TO_STRING(address->family, &address->in_addr_peer) : NULL;
 
+        const char *broadcast = (address->family == AF_INET && in4_addr_is_set(&address->broadcast)) ?
+                IN4_ADDR_TO_STRING(&address->broadcast) : NULL;
+
         (void) address_flags_to_string_alloc(address->flags, address->family, &flags_str);
         (void) route_scope_to_string_alloc(address->scope, &scope_str);
 
-        log_link_debug(link, "%s %s address (%s): %s%s%s/%u (valid %s, preferred %s), flags: %s, scope: %s",
+        log_link_debug(link, "%s %s address (%s): %s%s%s/%u%s%s (valid %s, preferred %s), flags: %s, scope: %s%s%s",
                        str, strna(network_config_source_to_string(address->source)), strna(state),
                        IN_ADDR_TO_STRING(address->family, &address->in_addr),
                        peer ? " peer " : "", strempty(peer), address->prefixlen,
+                       broadcast ? " broadcast " : "", strempty(broadcast),
                        FORMAT_LIFETIME(address->lifetime_valid_usec),
                        FORMAT_LIFETIME(address->lifetime_preferred_usec),
-                       strna(flags_str), strna(scope_str));
+                       strna(flags_str), strna(scope_str),
+                       address->family == AF_INET ? ", label: " : "",
+                       address->family == AF_INET ? strna(address->label) : "");
 }
 
 static int address_set_netlink_message(const Address *address, sd_netlink_message *m, Link *link) {
@@ -962,12 +1029,24 @@ int link_drop_foreign_addresses(Link *link) {
         ORDERED_HASHMAP_FOREACH(address, link->network->addresses_by_section) {
                 Address *existing;
 
+                if (address_get(link, address, &existing) < 0)
+                        continue;
+
                 /* On update, the kernel ignores the address label and broadcast address. Hence we need to
                  * distinguish addresses with different labels or broadcast addresses. Thus, we need to check
                  * the existing address with address_equal(). Otherwise, the label or broadcast address
                  * change will not be applied when we reconfigure the interface. */
-                if (address_get(link, address, &existing) >= 0 && address_equal(address, existing))
-                        address_unmark(existing);
+                if (address_is_static_null(address)) {
+                        /* If the static configuration has null address, then the broadcast address will be
+                         * determined automatically. Hence, here we need to compare only address label. */
+                        if (address->family == AF_INET && !streq_ptr(address->label, existing->label))
+                                continue;
+
+                } else if (!address_equal(address, existing))
+                        continue;
+
+                /* Found matching static configuration. Keep the existing address. */
+                address_unmark(existing);
         }
 
         /* Finally, remove all marked addresses. */
@@ -1027,10 +1106,8 @@ static int address_acquire(Link *link, const Address *original, Address **ret) {
         assert(ret);
 
         /* Something useful was configured? just use it */
-        if (in_addr_is_set(original->family, &original->in_addr)) {
-                *ret = NULL;
-                return 0;
-        }
+        if (in_addr_is_set(original->family, &original->in_addr))
+                return address_dup(original, ret);
 
         /* The address is configured to be 0.0.0.0 or [::] by the user?
          * Then let's acquire something more useful from the pool. */
@@ -1053,7 +1130,7 @@ static int address_acquire(Link *link, const Address *original, Address **ret) {
         na->in_addr = in_addr;
 
         *ret = TAKE_PTR(na);
-        return 1;
+        return 0;
 }
 
 int address_configure_handler_internal(sd_netlink *rtnl, sd_netlink_message *m, Link *link, const char *error_msg) {
@@ -1181,79 +1258,54 @@ int link_request_address(
                 address_netlink_handler_t netlink_handler,
                 Request **ret) {
 
-        Address *acquired, *existing = NULL;
+        _unused_ _cleanup_(address_freep) Address *address_will_be_freed = NULL;
+        Address *existing;
         int r;
 
         assert(link);
         assert(address);
         assert(address->source != NETWORK_CONFIG_SOURCE_FOREIGN);
 
-        r = address_acquire(link, address, &acquired);
-        if (r < 0)
-                return log_link_warning_errno(link, r, "Failed to acquire an address from pool: %m");
-        if (r > 0) {
-                if (consume_object)
-                        address_free(address);
+        if (consume_object)
+                address_will_be_freed = address;
 
-                address = acquired;
-                consume_object = true;
-        }
-
-        if (address_needs_to_set_broadcast(address, link)) {
-                if (!consume_object) {
-                        Address *a;
-
-                        r = address_dup(address, &a);
-                        if (r < 0)
-                                return r;
-
-                        address = a;
-                        consume_object = true;
-                }
-
-                address_set_broadcast(address, link);
-        }
-
-        (void) address_get(link, address, &existing);
-
-        if (address->lifetime_valid_usec == 0) {
-                if (consume_object)
-                        address_free(address);
-
-                /* The requested address is outdated. Let's remove it. */
-                return address_remove_and_drop(existing);
-        }
-
-        if (!existing) {
+        if (address_get(link, address, &existing) < 0) {
                 _cleanup_(address_freep) Address *tmp = NULL;
 
-                if (consume_object)
-                        tmp = address;
-                else {
-                        r = address_dup(address, &tmp);
-                        if (r < 0)
-                                return r;
-                }
+                if (address->lifetime_valid_usec == 0)
+                        /* The requested address is outdated. Let's ignore the request. */
+                        return 0;
+
+                r = address_acquire(link, address, &tmp);
+                if (r < 0)
+                        return log_link_warning_errno(link, r, "Failed to acquire an address from pool: %m");
+
+                address_set_broadcast(tmp, link);
 
                 /* Consider address tentative until we get the real flags from the kernel */
                 tmp->flags |= IFA_F_TENTATIVE;
 
                 r = address_add(link, tmp);
                 if (r < 0)
-                        return r;
+                        return log_link_warning_errno(link, r, "Failed to save requested address: %m");
 
                 existing = TAKE_PTR(tmp);
         } else {
+                if (address->lifetime_valid_usec == 0)
+                        /* The requested address is outdated. Let's remove it. */
+                        return address_remove_and_drop(existing);
+
                 r = address_equalify(existing, address);
                 if (r < 0)
-                        return r;
+                        return log_link_warning_errno(link, r, "Failed to equalify saved address with the requested one: %m");
+
+                address_set_broadcast(existing, link);
+
                 existing->source = address->source;
                 existing->provider = address->provider;
                 existing->duplicate_address_detection = address->duplicate_address_detection;
                 existing->lifetime_valid_usec = address->lifetime_valid_usec;
                 existing->lifetime_preferred_usec = address->lifetime_preferred_usec;
-                if (consume_object)
-                        address_free(address);
         }
 
         r = ipv4acd_configure(existing);
@@ -1263,8 +1315,8 @@ int link_request_address(
         log_address_debug(existing, "Requesting", link);
         r = link_queue_request_safe(link, REQUEST_TYPE_ADDRESS,
                                     existing, NULL,
-                                    address_hash_func,
-                                    address_compare_func,
+                                    address_kernel_hash_func,
+                                    address_kernel_compare_func,
                                     address_process_request,
                                     message_counter, netlink_handler, ret);
         if (r < 0)
@@ -1351,8 +1403,8 @@ void address_cancel_request(Address *address) {
                 .link = address->link,
                 .type = REQUEST_TYPE_ADDRESS,
                 .userdata = address,
-                .hash_func = (hash_func_t) address_hash_func,
-                .compare_func = (compare_func_t) address_compare_func,
+                .hash_func = (hash_func_t) address_kernel_hash_func,
+                .compare_func = (compare_func_t) address_kernel_compare_func,
         };
 
         request_detach(address->link->manager, &req);
@@ -1715,9 +1767,10 @@ int config_parse_address(
         n->family = f;
         n->prefixlen = prefixlen;
 
-        if (streq(lvalue, "Address"))
+        if (streq(lvalue, "Address")) {
                 n->in_addr = buffer;
-        else
+                n->requested_as_null = !in_addr_is_set(n->family, &n->in_addr);
+        } else
                 n->in_addr_peer = buffer;
 
         TAKE_PTR(n);
@@ -1755,6 +1808,12 @@ int config_parse_label(
                 return 0;
         }
 
+        if (isempty(rvalue)) {
+                n->label = mfree(n->label);
+                TAKE_PTR(n);
+                return 0;
+        }
+
         if (!address_label_valid(rvalue)) {
                 log_syntax(unit, LOG_WARNING, filename, line, 0,
                            "Interface label is too long or invalid, ignoring assignment: %s", rvalue);
@@ -2046,6 +2105,36 @@ int config_parse_address_netlabel(
         return 0;
 }
 
+static void address_section_adjust_broadcast(Address *address) {
+        assert(address);
+        assert(address->section);
+
+        if (!in4_addr_is_set(&address->broadcast))
+                return;
+
+        if (address->family == AF_INET6)
+                log_warning("%s: broadcast address is set for an IPv6 address. "
+                            "Ignoring Broadcast= setting in the [Address] section from line %u.",
+                            address->section->filename, address->section->line);
+        else if (address->prefixlen > 30)
+                log_warning("%s: broadcast address is set for an IPv4 address with prefix length larger than 30. "
+                            "Ignoring Broadcast= setting in the [Address] section from line %u.",
+                            address->section->filename, address->section->line);
+        else if (in4_addr_is_set(&address->in_addr_peer.in))
+                log_warning("%s: broadcast address is set for an IPv4 address with peer address. "
+                            "Ignoring Broadcast= setting in the [Address] section from line %u.",
+                            address->section->filename, address->section->line);
+        else if (!in4_addr_is_set(&address->in_addr.in))
+                log_warning("%s: broadcast address is set for an IPv4 address with null address. "
+                            "Ignoring Broadcast= setting in the [Address] section from line %u.",
+                            address->section->filename, address->section->line);
+        else
+                /* Otherwise, keep the specified broadcast address. */
+                return;
+
+        address->broadcast.s_addr = 0;
+}
+
 static int address_section_verify(Address *address) {
         if (section_is_invalid(address->section))
                 return -EINVAL;
@@ -2061,16 +2150,7 @@ static int address_section_verify(Address *address) {
 
         assert(IN_SET(address->family, AF_INET, AF_INET6));
 
-        if (in4_addr_is_set(&address->broadcast) &&
-            (address->family == AF_INET6 || address->prefixlen > 30 ||
-             in_addr_is_set(address->family, &address->in_addr_peer))) {
-                log_warning("%s: broadcast address is set for an IPv6 address, "
-                            "an IPv4 address with peer address, or with prefix length larger than 30. "
-                            "Ignoring Broadcast= setting in the [Address] section from line %u.",
-                            address->section->filename, address->section->line);
-
-                address->broadcast.s_addr = 0;
-        }
+        address_section_adjust_broadcast(address);
 
         if (address->family == AF_INET6 && address->label) {
                 log_warning("%s: address label is set for IPv6 address in the [Address] section from line %u. "
