@@ -63,7 +63,7 @@ static bool ignore_proc(pid_t pid, bool warn_rootfs) {
                 return false;
 
         if (warn_rootfs &&
-            pid_from_same_root_fs(pid) == 0) {
+            pid_from_same_root_fs(pid) > 0) {
 
                 _cleanup_free_ char *comm = NULL;
 
